from __future__ import annotations
import random
from .element import Element
from selenium.webdriver.chrome.webdriver import WebDriver
from pycollection import Collection

class ElementCollection(Collection):

    def __init__(self, items, driver:WebDriver):
        super().__init__(items)
        self.driver = driver

    def item(self, item) -> Element:
        return Element(item, self.driver)


from .element_collection import ElementCollection

from .element import Element
from selenium.webdriver.support import expected_conditions as EC
from datetime import datetime
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException

class ElementFinder:

    def __init__(self, locator: dict, driver, element=None):
        self.locator = locator
        self.driver = driver
        self.element = element
        self.wait = 0

    def until(self, seconds):
        self.wait = seconds
        return self

    def get(self):
        try:
            scope = self.element if self.element else self.driver

            return ElementCollection(
                WebDriverWait(scope, self.wait).until(
                    EC.presence_of_all_elements_located(self.locator)
                ),
                self.driver
            )
        except TimeoutException:
            self.driver.get_screenshot_as_file( "images/errors/"+datetime.now().strftime("%d-%b-%Y (%H:%M:%S.%f).png") )
            raise exceptions.element_not_found_exception.ElementNotFoundException(f"the element ({','.join(self.locator)}) was not found")

    def first(self):
        try:

            scope = self.element if self.element else self.driver

            return Element(
                WebDriverWait(scope, self.wait).until(
                    EC.element_to_be_clickable(self.locator)
                ),
                self.driver
            )
        except TimeoutException:
            self.driver.get_screenshot_as_file( "images/errors/"+datetime.now().strftime("%d-%b-%Y (%H:%M:%S.%f).png") )
            raise exceptions.element_not_found_exception.ElementNotFoundException(f"the element ({','.join(self.locator)}) was not found")

    @staticmethod
    def where_xpath(xpath:str, driver, element=None):
        return ElementFinder( 
            (By.XPATH, xpath), 
            driver , 
            element
        )

    @staticmethod
    def where_id(id:str, driver, element=None):
        return ElementFinder( 
            (By.ID, id), 
            driver, 
            element
        )

    @staticmethod
    def where_name(name:str, driver, element=None):
        return ElementFinder( 
            (By.NAME, name), 
            driver , 
            element
        )

    @staticmethod
    def where_contain_text(text:str, driver, element=None):
        return ElementFinder.where_xpath( 
            f"//*[contains(text(), '{text}')]", 
            driver , 
            element
        )

    @staticmethod
    def where_class_name(name:str, driver, element=None):
        return ElementFinder( 
            (By.CLASS_NAME, name), 
            driver , 
            element
        )

    @staticmethod
    def where_tag_name(name:str, driver, element=None):
        return ElementFinder( 
            (By.TAG_NAME, name), 
            driver, 
            element
        )


from __future__ import annotations
from typing import TYPE_CHECKING
from selenium.webdriver.common.by import By

from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.common.keys import Keys
import scrappier

class Element():
    """
    A class that represents a DOM element
    """

    def __init__(self, element: WebElement, driver: WebDriver):
        """

        Args:
            element (WebElement): web element to wrap
            driver (WebDriver): web driver
        """

        self.driver = driver
        self.element = element

    def enter(self):
        self.element.send_keys(Keys.RETURN)

    def type(self, text:str):
        """
        Types an specific text in an input field

        """

        self.element.send_keys(text)

    def text(self) -> str:
        """gets the inner text of the element

        Returns:
            str: inner text of the element
        """

        return self.element.text

    def html(self) -> str:
        """gets the inner html of the element

        Returns:
            str: the inner html of the element
        """

        return self.element.get_attribute('innerHTML')

    def attribute(self, name:str) -> str:
        """gets the attribute's value

        Args:
            name (str): name of the attribute to search

        Returns:
            str: attribute's value
        """

        return self.element.get_attribute(name)

    def click(self):
        """
        clicks the element
        """

        self.element.click()

    def clear(self):
        """
        clear the input field
        """

        self.element.clear()

    def children(self) -> scrappier.element_collection.ElementCollection:
        """
        gets a collection of the next children elements
        """
        return scrappier.element_collection.ElementCollection(
            self.element.find_elements(By.XPATH,"./child::*"),
            self.driver
        )

    def where_tag_name(self, name:str) -> ElementFinder:
        """
        find a subelement with the tag name specified
        """

        return ElementFinder.where_tag_name(
            name,
            self.driver,
            self.element
        )

    def where_attribute(self, attribute:str, value:str) -> ElementFinder:
        """find a subelement with the attribute and value specified

        Args:
            attribute (str): the attribute of the element
            value (str): the value of the attribute

        Returns:
            ElementFinder
        """

        return ElementFinder.where_xpath(
            f".//*[@{attribute}='{value}']",
            self.driver,
            self.element
        )


from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options
from selenium import webdriver

from scrappier.element_collection import ElementCollection
from scrappier.element_finder import ElementFinder

from .element import Element
from selenium.webdriver.support import expected_conditions as EC
from datetime import datetime
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException

class Browser:
    
    options = [
        '--headless',
        '--disable-gpu',
        '--incognito'
    ]

    driver = None

    service = Service('/usr/bin/chromedriver')

    def __init__(self):
        self.build()
        self.resize(1400, 1000)

    def width(self) -> int:
        return self.driver.get_window_size()["width"]

    def build(self):

        options = Options()

        for option in self.options:
            options.add_argument(option)

        self.driver = webdriver.Chrome(
            service=self.service,
            options=options
        )

        self.driver.delete_all_cookies()

    def resize(self, width:int, height:int):
        if self.driver:
            self.driver.set_window_size(width,height)

    def webdriver(self):
        return self.driver

    def screenshot(self, path:str):
        self.driver.get_screenshot_as_file(f"{path}")

    def snapshot(self):
        path = f"{datetime.now().strftime('%d-%m-%Y-%H-%M-%S')}.png"

        self.screenshot(path)

    def wait(self, seconds:int):
        self.driver.implicitly_wait(seconds)

    def visit(self, url:str) -> None:
        self.driver.get(url)

    def where_xpath(self, xpath:str) -> 'ElementFinder':
        return ElementFinder(
            locator = (By.XPATH, xpath),
            driver = self.driver
        )

    def where_id(self, id:str) -> 'ElementFinder':
        return ElementFinder(
            (By.ID, id),
            self.driver
        )

    def where_name(self, name:str) -> 'ElementFinder':
        return ElementFinder(
            (By.NAME, name),
            self.driver
        )

    def where_contain_text(self, text:str) -> 'ElementFinder':
        return ElementFinder.where_xpath(
            f"//*[contains(text(), '{text}')]",
            self.driver
        )

    def where_class_name(self, name:str) -> 'ElementFinder':
        return ElementFinder(
            (By.CLASS_NAME, name),
            self.driver
        )

    def where_tag_name(self, name:str) -> 'ElementFinder':
        return ElementFinder(
            (By.TAG_NAME, name),
            self.driver
        )

    def where_attribute(self, attribute, value) -> 'ElementFinder':
        return ElementFinder(
            (By.XPATH, f"//*[@{attribute}='{value}']"),
            self.driver
        )


from setuptools import setup, find_packages
import codecs
import os

VERSION = '1.0.1'
DESCRIPTION = 'Scrapper for chrome'

with codecs.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# Setting up
setup(
    name="scrappier",
    version=VERSION,
    author="Cristian Guzmán",
    author_email="<cristian.guzman.contacto@gmail.com>",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=["typing","selenium", "pycollection"],
    keywords=['python', 'scrapper', 'web', 'scrapping', 'selenium', 'scrappier', 'browser', 'chrome', 'datetime'],
    classifiers=[]
)